<?php

use Endroid\QrCode\Encoding\Encoding;
use Endroid\QrCode\Logo\Logo;
use Endroid\QrCode\Writer\PngWriter;
use Xamax\Wordpress\XamaxPayment;
use Endroid\QrCode\QrCode;
use Endroid\QrCode\ErrorCorrectionLevel;

if (!defined('ABSPATH')) {
	exit('You must not access this file directly');
}

class Xamax_Payment_Gateway extends WC_Payment_Gateway {
	/**
	 * API Key
	 *
	 * @var string
	 */
	public $sandbox_api_key;

	/**
	 * API Key
	 *
	 * @var string
	 */
	public $production_api_key;

	/**
	 * The environment of transactions
	 *
	 * @var string
	 */
	public $environment;

	/**
	 * Name of the payment method
	 *
	 * @var string
	 */
	public $name_of_payment_method;

	/**
	 * The description of payment method
	 *
	 * @var string
	 */
	public $description;

	/**
	 * The instruction for users
	 *
	 * @var string
	 */
	public $instruction;

	/**
	 * Order status after successful payment
	 *
	 * @var string
	 */
	public $order_status;

	/**
	 * Order conversion rate for USD / Store currency
	 *
	 * @var float
	 */
	public $usd_conversion_rate;

	public function __construct() {
		$this->id = 'xamax_payment';
		$this->icon = XAMAX_PAYMENT_PLUGIN_URL . '/assets/images/xamax_logo.png';
		$this->method_title = esc_html__('Xamax Payment Gateway', 'xamax-payment');
		$this->method_description = esc_html__('Expand your payment options by accepting cryptocurrency payments (BTC, BCH, ETH, and Stable Coins) without risk or price fluctuations.', 'xamax-payment');
		$this->name_of_payment_method = esc_html__('Xamax', 'xamax-payment');

		$this->supports = ['products'];
		$this->has_fields = false;

		$this->init_form_fields();
		$this->init_settings();

		add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);
		add_action('admin_enqueue_scripts', [$this, 'admin_scripts']);
		add_action('woocommerce_api_' . $this->id, [$this, 'webhook']);
		add_action('valid-xamax-standard-request', [$this, 'successful_request']);
		add_action('woocommerce_receipt_' . $this->id, [$this, 'receipt_page']);

		if (!$this->is_valid_for_use()) {
			$this->enabled = false;
		}
	}

	protected function getXamaxAdapter(): XamaxPayment {
		return new XamaxPayment(
			$this->get_option('environment'),
			$this->get_option('production_api_key'),
			$this->get_option('sandbox_api_key')
		);
	}

	public function generate_form_pay(WC_Order $order, XamaxPayment $xamaxAdapter): string {
		$payCurrency = sanitize_text_field($_GET['currency'] ?? 'btc');

		$totalAmount = number_format(
			$order->get_total() / $this->get_option('usd_conversion_rate'), 2, '.', ''
		);

		$invoice = $xamaxAdapter->createInvoice(
			$order->get_id(),
			$payCurrency,
			'usd',
			$totalAmount
		);

		if (!isset($invoice['walletAddress'])) {
			return $this->generate_form_error($invoice);
		}

		$order->update_meta_data('xamax_order_bearer_token', "Bearer {$xamaxAdapter->getAccessToken()}");
		$order->save();

		ob_start();

		$writer = new PngWriter();

		// Create QR code
		$qrCode = QrCode::create($invoice['walletAddress'])
			->setEncoding(new Encoding('UTF-8'))
			->setErrorCorrectionLevel(new ErrorCorrectionLevel\ErrorCorrectionLevelLow())
			->setSize(300)
			->setMargin(10)
			->setRoundBlockSizeMode(new Endroid\QrCode\RoundBlockSizeMode\RoundBlockSizeModeMargin())
			->setForegroundColor(new Endroid\QrCode\Color\Color(0, 0, 0))
			->setBackgroundColor(new Endroid\QrCode\Color\Color(255, 255, 255));

		// Create generic logo
		$logo = Logo::create(XAMAX_PAYMENT_PLUGIN_PATH . '/assets/images/xamax_logo.png')
			->setResizeToWidth(50)
			->setPunchoutBackground(true);

		$result = $writer->write($qrCode, $logo);

		$imageString = base64_encode($result->getString());

		ob_end_clean();

		$completeUrl = $order->get_checkout_order_received_url();
		$cancelUrl = $order->get_cancel_order_url();
		$payload = getIconAndLabelForCurrency($invoice['codeRequested']);
		$orderNumber = $invoice['txId'];
		$url = $xamaxAdapter->getUrl();
		$authToken = $xamaxAdapter->getAccessToken();

		return render('payment-form', compact(
			'imageString',
			'payload',
			'completeUrl',
			'cancelUrl',
			'invoice',
			'order',
			'orderNumber',
			'url',
			'authToken'
		));
	}

	protected function is_valid_for_use(): bool {
		return get_option('woocommerce_currency') == 'USD';
	}

	public function admin_scripts() {
		wp_enqueue_script('xamax-admin-script', XAMAX_PAYMENT_PLUGIN_URL . 'assets/js/admin.js', ['jquery'], XAMAX_PAYMENT_VERSION, true);
	}

	public function payment_fields() {
		if ($this->instruction) {
			echo esc_html(wptexturize($this->instruction));
		}
	}

	public function validate_fields(): bool {
		return true;
	}

	public function process_payment($order_id): array {
		$order = new WC_Order($order_id);

		return [
			'result' => 'success',
			'redirect' => add_query_arg('order-pay', $order_id, add_query_arg('key', $order->order_key, $order->get_checkout_payment_url(true))),
		];
	}

	public function generate_form_error(array $invoice): string {
		$out = '<ul>';
		foreach ($invoice as $key => $value) {
			if (!is_array($value) && !is_object($value)) {
				$out .= '<li>' . esc_html__($key, 'xamax-payment') . ': <strong>' . esc_html__($value, 'xamax-payment') . '</strong></li>';
			}
		}
		$out .= '</ul>';

		return $out;
	}

	public function generate_form_currency_select(WC_Order $order, XamaxPayment $xamaxAdapter): string {
		$getArray = $_GET;
		$currencies = $xamaxAdapter->getCryptoCurrencies()['blockchains'];

		$hidden_args = [];
		$payloads = [];

		foreach ($currencies as $currency) {
			if ('bsc' != $currency['code']) {
				$payloads[] = getIconAndLabelForCurrency($currency['code']) + ['currency' => $currency['code']];
			}
			if (isset($currency['tokens'])) {
				foreach ($currency['tokens'] as $token) {
					$payloads[] = getIconAndLabelForCurrency($token['code']) + ['currency' => $token['code']];
				}
			}
		}

		usort($payloads, 'sortByPriority');

		$currencies_args = array_map(function (array $data) {
			return '
				<div style="display: flex; width: 200px; margin-bottom: 5px; cursor: pointer">
					<input style="cursor: pointer" id="form-input-' . esc_attr($data['currency']) . '" type="radio" name="currency" value="' . esc_attr($data['currency']) . '"/>
					<label for="form-input-' . esc_attr($data['currency']) . '" style="flex: 1; margin: 0; cursor: pointer">
						<span style="display:flex; justify-content: space-between;">' . esc_html($data['label']) . '<img width="24px" height="24px" src="' . esc_url($data['icon']) . '" alt=""></span>
					</label>
				</div>
			';
		}, $payloads);

		foreach ($getArray as $arg => $value) {
			$hidden_args[] = '<input type="hidden" name="' . esc_attr($arg) . '" value="' . esc_attr($value) . '"/>';
		}

		return
			'<form action="" method="GET" id="xamax_payment_form">' . "\n" .
			implode("\n", $currencies_args) .
			implode("\n", $hidden_args) .
			'<br>' .
			'<input type="submit" class="btn-lex button wp-element-button product_type_simple add_to_cart_button ajax_add_to_cart" id="submit_xamax_payment_form" value="' . esc_html__('Pay now', 'xamax-payment') . '" /> <a class="button cancel btn-lex shadow" href="' . esc_url($order->get_cancel_order_url()) . '">' . esc_html__('Return to cart without payment', 'xamax-payment') . '</a>' . "\n" .
			'</form>';
	}

	public function webhook(): string {
		$request = json_decode(file_get_contents('php://input'), true);
		$headers = getallheaders();

		if (
			isset($request['txId'], $request['status']) &&
			in_array($request['status'], ['transaction_status_processed', 'transaction_status_confirmed'])
		) {
			@ob_clean();

			$orderId = (int) $request['txId'];
			$order = new WC_Order($orderId);

			$validated = validateHeaderBearer($headers['Authorization'], $request, $this->get_option('environment'));

			if ($validated) {
				WC()->cart->empty_cart();
				$order->update_status($this->get_option('order_status'), esc_html__('Order was paid success', 'xamax-payment'));
				do_action('valid-xamax-standard-request', $request);
				wp_redirect($this->get_return_url($order));

				return 'ok';
			}
		} elseif (
			isset($request['txId'], $request['status']) &&
			'transaction_status_pending' === $request['status'] 
		) {
			$orderId = (int) $request['txId'];
			$order = new WC_Order($orderId);
			$order->update_status('processing', esc_html__('Waiting for payment by Xamax', 'xamax-payment'));

			return 'ok';
		}

		$orderId = (int) $request['txId'];
		$order = new WC_Order($orderId);
		$order->update_status('failed', esc_html__('Payment canceled', 'xamax-payment'));
		wp_redirect($order->get_cancel_order_url());

		return 'fail';
	}

	public function successful_request($get) {
		$orderID = (int) $get['txId'];
		$order = new WC_Order($orderID);
		// Check order not already completed
		if ('completed' === $order->status) {
			return;
		}
		// Payment completed
		$order->add_order_note(esc_html__('Payment made with success!', 'xamax-payment'));
		$order->update_status($this->get_option('order_status'), esc_html__('Order was paid success', 'xamax-payment'));
		$order->payment_complete();
	}

	public function init_form_fields() {
		$this->form_fields = apply_filters('woo_xamax_payment', [
			'enabled' => [
				'title' => esc_html__('Enable/Disable', 'xamax-payment'),
				'type' => 'checkbox',
				'label' => esc_html__('Enable Xamax Payment Gateway', 'xamax-payment'),
				'default' => 'no',
			],
			'title' => [
				'title' => esc_html__('Name of payment method', 'xamax-payment'),
				'type' => 'text',
				'description' => esc_html__('User see this title in order checkout.', 'xamax-payment'),
				'default' => esc_html__('Xamax', 'xamax-payment'),
			],
			'production_api_key' => [
				'title' => esc_html__('Production API Key', 'xamax-payment'),
				'type' => 'text',
				'description' => esc_html__('Check Xamax https://my.xamax.io/integrations/documentation/crypto-processing/get-api-key for more information', 'xamax-payment'),
				'default' => '',
			],
			'sandbox_api_key' => [
				'title' => esc_html__('Sandbox API Key', 'xamax-payment'),
				'type' => 'text',
				'description' => esc_html__('Check Xamax https://my.sandbox.xamax.io/integrations/documentation/crypto-processing/get-api-key for more information', 'xamax-payment'),
				'default' => '',
			],
			'environment' => [
				'title' => esc_html__('Xamax environment', 'xamax-payment'),
				'type' => 'select',
				'options' => [
					'production' => esc_html__('production', 'xamax-payment'),
					'sandbox' => esc_html__('sandbox', 'xamax-payment'),
				],
				'default' => 'sandbox',
				'description' => esc_html__('Set environment for payment gateway', 'xamax-payment'),
			],
			'usd_conversion_rate' => [
				'title' => esc_html__('Order amount USD conversion rate', 'xamax-payment'),
				'type' => 'text',
				'description' => esc_html__('Please insert order conversion rate for USD / Store currency', 'xamax-payment'),
				'default' => '1.00',
			],
			'order_status' => [
				'title' => esc_html__('Order status', 'xamax-payment'),
				'type' => 'select',
				'options' => wc_get_order_statuses(),
				'description' => esc_html__('Setup order status after successful payment', 'xamax-payment'),
			],
			'description' => [
				'title' => esc_html__('Description', 'xamax-payment'),
				'type' => 'textarea',
				'description' => esc_html__('Description of payment method which user can see in your site.', 'xamax-payment'),
				'default' => esc_html__('Payment with Xamax payment gateway', 'xamax-payment'),
			],
			'instructions' => [
				'title' => esc_html__('Instructions', 'xamax-payment'),
				'type' => 'textarea',
				'description' => esc_html__('Instructions which can be added on the thank-you-payment page.', 'xamax-payment'),
				'default' => esc_html__('Payment with Xamax payment gateway! Thank you for your payment', 'xamax-payment'),
			],
		]);
	}

	public function admin_options() {
		?>
		<h3><?php esc_html_e('XAMAX', 'xamax-payment'); ?></h3>
		<p><?php esc_html_e('Please select payment crypto currency.', 'xamax-payment'); ?></p>
		<?php if ( $this->is_valid_for_use() ) : ?>
			<table class="form-table">
				<?php
				// Generate the HTML For the settings form.
				$this->generate_settings_html();
				?>
			</table><!--/.form-table-->
		<?php else : ?>
			<div class="inline error">
				<p>
					<strong><?php esc_html_e('Gateway is disable', 'xamax-payment'); ?></strong>:
					<?php esc_html_e('XAMAX does not supported currencies of store.', 'xamax-payment'); ?>
				</p>
			</div>
		<?php
		endif;
	}

	public function receipt_page($order) {
		if (!is_object($order)) {
			$order = new WC_Order((int) $order);
		}

		$xamaxAdapter = $this->getXamaxAdapter();

		$amount = number_format(
			$order->get_total() / $this->get_option('usd_conversion_rate'), 2, '.', ''
		);

		echo '<div class="xamax-payment-block-info">';

		if (!isset($_GET['currency'])) {
			echo '<p style="font-weight:500;"><i class="fal fa-info-circle"></i><strong>' .
				esc_html__(
					'You must pay approx: ',
					'xamax-payment'
				) . esc_html($amount) . ' USD</strong></p>';

			echo '<p style="font-weight:500;"><i class="fal fa-info-circle"></i> ' .
				esc_html__(
					'Thank you for your order, please click the button below to pay for your order by crypto currency.',
					'xamax-payment'
				) . '</p>';
			echo '<p style="font-weight:500;"><i class="fal fa-info-circle"></i> ' .
				esc_html__('Please select a crypto currency to pay for your order:', 'xamax-payment') . '</p>';
			$out = $this->generate_form_currency_select($order, $xamaxAdapter);
		} else {
			$out = $this->generate_form_pay($order, $xamaxAdapter);
		}

		echo $out;

		echo '</div>';

		return null;
	}
}
