<?php

namespace Xamax\Wordpress;

use Exception;

/**
 * Adapter for using Xamax services
 *
 * @class XamaxPayment
 * @docs https://api.xamax.io/swagger#/
 */
class XamaxPayment {
	private const PRODUCTION_ENDPOINT = 'https://api.xamax.io/';
	private const SANDBOX_ENDPOINT = 'https://api.sandbox.xamax.io/';
	private $url;
	private $apiKey;
	protected $accessToken;
	protected $environment;

	/**
	 * Constructor
	 *
	 * @throws Exception
	 */
	public function __construct(
		string $environment,
		string $productionApiKey,
		string $sandboxApiKey
	) {
		$this->environment = $environment;
		switch ($environment) {
			case 'production':
				$this->url    = self::PRODUCTION_ENDPOINT;
				$this->apiKey = $productionApiKey;
				break;
			default:
				$this->url    = self::SANDBOX_ENDPOINT;
				$this->apiKey = $sandboxApiKey;
		}

		$this->accessToken = $this->getRefreshToken($this->apiKey);
	}

	public function getAccessToken(): string {
		return $this->accessToken;
	}

	public function getUrl(): string {
		return $this->url;
	}

	/**
	 * Method for getting refresh token
	 *
	 * @docs https://api.xamax.io/swagger#/Authentication/Api_AuthRefresh
	 * @throws Exception
	 */
	protected function getRefreshToken(string $apiKey): string {
		$curl = curl_init();

		curl_setopt_array($curl, [
			CURLOPT_URL => $this->url . 'v1/auth/refresh',
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_POST => true,
			CURLOPT_POSTFIELDS => json_encode(['refresh_token' => $apiKey]),
			CURLOPT_HTTPHEADER => [
				'Accept: application/json',
				'Content-Type: application/json',
			],
		]);

		$response = curl_exec($curl);
		$httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
		$err = curl_error($curl);
		curl_close($curl);

		if (400 <= $httpCode || $err) {
			throw new Exception('Failed to get access token');
		}

		return json_decode($response)->access_token;
	}

	/**
	 * Method for creating invoices
	 *
	 * @docs https://api.xamax.io/swagger#/Transaction/Api_TransactionCreateInvoice
	 * @throws Exception
	 */
	public function createInvoice(
		int $orderId,
		string $cryptoCurrency,
		string $fiatCurrency,
		float $amount
	): array {
		$curl = curl_init();

		curl_setopt_array($curl, [
			CURLOPT_URL => $this->url . 'v1/transaction/invoice',
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_POST => true,
			CURLOPT_POSTFIELDS => json_encode([
				'txId' => $orderId,
				'code' => $cryptoCurrency,
				'fiat' => [
					'amount' => $amount,
					'currency' => $fiatCurrency,
				],
			]),
			CURLOPT_HTTPHEADER => [
				'Accept: application/json',
				'Authorization: Bearer ' . $this->accessToken,
				'Content-Type: application/json',
			],
		]);

		$response = curl_exec($curl);
		$err = curl_error($curl);
		curl_close($curl);

		if ($err) {
			throw new Exception('Failed to create invoice');
		}

		return json_decode($response, true);
	}

	/**
	 * Method for getting list of supported cryptocurrencies
	 *
	 * @docs https://api.xamax.io/swagger#/Api/Api_ConfigGet
	 * @throws Exception
	 */
	public function getCryptoCurrencies(): array {
		$curl = curl_init();

		curl_setopt_array($curl, [
			CURLOPT_URL => $this->url . '.config/processing.json',
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_HTTPHEADER => [
				'Accept: application/json',
				'Content-Type: application/json',
			],
		]);

		$response = curl_exec($curl);
		$httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
		$err = curl_error($curl);
		curl_close($curl);

		if (400 <= $httpCode || $err) {
			throw new Exception('Failed to get crypto currencies');
		}

		return json_decode($response, true);
	}

	/**
	 * Method for getting the list of JWKS keys for validation processing callback
	 *
	 * @docs  https://api.xamax.io/swagger#/Api/Api_JWKsGet
	 * @throws Exception
	 */
	public static function getOpenJwks(string $environment): array {
		$url = ( self::PRODUCTION_ENDPOINT === $environment ) ? self::PRODUCTION_ENDPOINT : self::SANDBOX_ENDPOINT;
		$curl = curl_init();

		curl_setopt_array($curl, [
			CURLOPT_URL => $url . '.well-known/jwks.json',
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_HTTPHEADER => [
				'Accept: application/json',
				'Content-Type: application/json',
			],
		]);

		$response = curl_exec($curl);
		$httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
		$err = curl_error($curl);
		curl_close($curl);

		if (400 <= $httpCode || $err) {
			throw new Exception('Failed to get open jwks');
		}

		return json_decode($response, true);
	}
}
